@@ -12,8 +12,8 @@ impl Kubectl {
         r#type: &str,
         name: &str,
         string_data: json::Value,
-    ) -> anyhow::Result<Secret> {
-        let secrets = self.namespaced_api::<Secret>(namespace).await?;
+    ) -> anyhow::Result<corev1::Secret> {
+        let secrets = self.namespaced_api::<corev1::Secret>(namespace).await?;
         let secret = secrets.get(name).await;
 
         let (secret, replace) = if let Ok(secret) = secret {
