@@ -14,6 +14,7 @@ use dialoguer::{theme, Confirm, Input, MultiSelect, Select};
 use futures::StreamExt;
 use itertools::Itertools;
 use serde::{de, Deserialize, Serialize};
+use statehub_cluster_operator::StatehubCluster;
 
 use crate::api;
 use crate::k8s;
@@ -120,6 +121,7 @@ enum Command {
             parse(try_from_str = parse_params),
             alias = "param",
             hide = true,
+            multiple_values(true),
         )]
         params: Vec<(String, String)>,
     },
@@ -359,11 +361,7 @@ enum Command {
         location: Location,
     },
 
-    #[clap(
-        about = "Set state availability grade",
-        display_order(40),
-        setting(clap::AppSettings::Hidden)
-    )]
+    #[clap(about = "Set state availability grade", display_order(40), hide(true))]
     SetAvailability,
 
     #[clap(about = "Set cluster as the state owner", display_order(40))]
@@ -386,7 +384,7 @@ enum Command {
         about = "Manually create new volume",
         visible_aliases = &["create-v", "cv"],
         display_order(50),
-        setting(clap::AppSettings::Hidden),
+        hide(true),
     )]
     CreateVolume {
         #[clap(help = "State name")]
@@ -403,7 +401,7 @@ enum Command {
         about = "Manually delete existing volume",
         visible_aliases = &["delete-v", "dv"],
         display_order(50),
-        setting(clap::AppSettings::Hidden),
+        hide(true),
     )]
     DeleteVolume {
         #[clap(help = "State name")]
@@ -418,7 +416,7 @@ enum Command {
         about = "Manually choose primary location for volume",
         visible_aliases = &["set-p", "sp"],
         display_order(50),
-        setting(clap::AppSettings::Hidden),
+        hide(true),
     )]
     SetPrimary {
         #[clap(help = "State name")]
@@ -474,7 +472,7 @@ enum Command {
         about = "Just register named cluster via API (no interaction with K8s cluster",
         visible_alias = "jrc",
         display_order(1000),
-        setting(clap::AppSettings::Hidden)
+        hide(true)
     )]
     JustRegisterCluster {
         #[clap(help = "Cluster name")]
@@ -496,7 +494,7 @@ enum Command {
         about = "Just unregister named cluster via API (no interaction with K8s cluster)",
         visible_aliases = &["jurc", "juc"],
         display_order(1000),
-        setting(clap::AppSettings::Hidden)
+        hide(true)
     )]
     JustUnregisterCluster {
         #[clap(help = "Cluster name")]
@@ -507,7 +505,7 @@ enum Command {
         about = "Save cluster token",
         visible_alias = "sct",
         display_order(1000),
-        setting(clap::AppSettings::Hidden)
+        hide(true)
     )]
     SaveClusterToken {
         #[clap(help = "Namespace name")]
@@ -522,7 +520,7 @@ enum Command {
         about = "Setup statehub configmap",
         visible_aliases = &["scm", "setup-cm"],
         display_order(1000),
-        setting(clap::AppSettings::Hidden),
+        hide(true)
     )]
     SetupConfigmap {
         #[clap(help = "cluster name")]
@@ -549,7 +547,7 @@ enum Command {
         about = "Install Statehub Cluster CRD",
         visible_aliases = &["icc", "crd"],
         display_order(1000),
-        setting(clap::AppSettings::Hidden),
+        hide(true)
     )]
     InstallClusterCrd {
         #[clap(help = "cluster name")]
@@ -571,7 +569,7 @@ enum Command {
         about = "Uninstall Statehub Cluster CRD",
         visible_aliases = &["ucc", "uncrd"],
         display_order(1000),
-        setting(clap::AppSettings::Hidden),
+        hide(true)
     )]
     UninstallClusterCrd {
         #[clap(help = "cluster name")]
@@ -588,28 +586,20 @@ enum Command {
         about = "List K8s namespaces",
         visible_alias = "list-ns",
         display_order(1000),
-        setting(clap::AppSettings::Hidden)
+        hide(true)
     )]
     ListNamespaces {
         #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
         context: Option<String>,
     },
 
-    #[clap(
-        about = "List K8s nodes",
-        display_order(1000),
-        setting(clap::AppSettings::Hidden)
-    )]
+    #[clap(about = "List K8s nodes", display_order(1000), hide(true))]
     ListNodes {
         #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
         context: Option<String>,
     },
 
-    #[clap(
-        about = "List K8s pods",
-        display_order(1000),
-        setting(clap::AppSettings::Hidden)
-    )]
+    #[clap(about = "List K8s pods", display_order(1000), hide(true))]
     ListPods {
         #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
         context: Option<String>,
@@ -619,7 +609,7 @@ enum Command {
         about = "List K8s node regions",
         visible_aliases = &["list-r", "lr"],
         display_order(1000),
-        setting(clap::AppSettings::Hidden),
+        hide(true)
     )]
     ListRegions {
         #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
@@ -1134,12 +1124,13 @@ impl StateHub {
         let kubectls = k8s::Kubectl::with_names_and_contexts(names, contexts)?;
 
         for kubectl in kubectls {
+            let name = kubectl.cluster_name();
             self.inform(format_args!(
-                "Updating cluster {} (kubeconfig context {})",
-                kubectl.cluster_name(),
+                "Updating cluster {name} (kubeconfig context {})",
                 kubectl.context()
             ));
-            self.setup_credentials_helper(&kubectl, &namespace, &aws, &azure)
+            let statehub_cluster = StatehubCluster::with_namespace(name, &namespace);
+            self.setup_credentials_helper(&kubectl, &statehub_cluster, &aws, &azure)
                 .await?;
         }
         Ok(())
@@ -1164,13 +1155,8 @@ impl StateHub {
             let check_cluster = kubectl.check_cluster().await;
             futures::pin_mut!(check_cluster);
             while let Some(item) = check_cluster.next().await {
-                match item {
-                    Ok(text) => self.inform(text),
-                    Err(text) => {
-                        self.inform(text);
-                        break;
-                    }
-                }
+                let text = item?;
+                self.inform(text);
             }
         }
         Ok(())
@@ -1427,16 +1413,10 @@ impl StateHub {
         let contexts = context.into_iter().collect();
         let kubectls = k8s::Kubectl::with_names_and_contexts(names, contexts)?;
         let default_state = default_state.as_deref().unwrap_or("");
-        let api = self.api.url("");
         for kubectl in kubectls {
-            kubectl
-                .store_configmap(
-                    &namespace,
-                    kubectl.cluster_name(),
-                    default_state,
-                    &api,
-                    &cleanup_grace,
-                )
+            let name = kubectl.cluster_name();
+            let statehub_cluster = StatehubCluster::with_namespace(name, &namespace);
+            self.setup_configmap_helper(&kubectl, &statehub_cluster, default_state, &cleanup_grace)
                 .await?;
         }
 
